package com.dxhy.erp.service.impl;

import cn.hutool.core.util.ReflectUtil;
import com.alibaba.fastjson.JSONObject;
import com.dxhy.common.constant.DbConstant;
import com.dxhy.common.datasource.config.DynamicContextHolder;
import com.dxhy.common.service.impl.MpBaseServiceImpl;
import com.dxhy.common.utils.R;
import com.dxhy.erp.dao.RequestRecordDao;
import com.dxhy.erp.entity.RequestRecord;
import com.dxhy.erp.enums.RequestRecordEnum;
import com.dxhy.erp.service.RequestRecordService;
import com.dxhy.erp.service.SNPushCheckRecordService;
import com.github.pagehelper.PageHelper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.lang.reflect.Method;
import java.util.Date;
import java.util.List;

@Slf4j
@Service
public class RequestRecordServiceImpl extends MpBaseServiceImpl<RequestRecordDao, RequestRecord> implements RequestRecordService {

    private final Integer MAX_TIMES = 10;

    @Resource
    RequestRecordDao requestRecordDao;

    @Resource
    private SNPushCheckRecordService snPushCheckRecordService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public int saveRecord(RequestRecord record, Integer failedTimes, Integer status, String responseDetail, Integer... failedReason) {
        DynamicContextHolder.push("business" + DbConstant.BUSINESS_WRITE);
        record.setFailedTimes(failedTimes);
        record.setStatus(status);
        record.setResponseDetail(responseDetail);
        if (failedReason.length > 0) {
            record.setFailedReason(failedReason[0]);
        }
        int insert = super.getBaseMapper().insert(record);
        return insert;
    }

    @Override
    public List<RequestRecord> getRecordList(String portName, Date startTime, Date endTime, Integer status, String methodName, Integer pageNumber, Integer PageSize) {
        DynamicContextHolder.push("business" + DbConstant.BUSINESS_READ);
        PageHelper.startPage(pageNumber, PageSize);
        List<RequestRecord> recordList = requestRecordDao.selectRecordList(portName, startTime, endTime, status, methodName);
        return recordList;
    }

    @Override
    public RequestRecord getBaseRequestRecord(String requestDetail, String portName, String invoiceCode, String invoiceNo, String methodName) {
        return RequestRecord.builder()
                .requestDetail(requestDetail)
                .portName(portName)
                .createTime(new Date())
                .invoiceCode(invoiceCode)
                .invoiceNo(invoiceNo)
                .methodName(methodName)
                .build();
    }

    @Override
    public ResponseEntity<?> reCall(RequestRecord requestRecord) {
        DynamicContextHolder.push("business" + DbConstant.BUSINESS_READ);
        RequestRecord dbRecord = requestRecordDao.selectById(requestRecord.getId());
        if (!dbRecord.getRequestDetail().equals(requestRecord.getRequestDetail())
                || !dbRecord.getResponseDetail().equals(requestRecord.getResponseDetail())
                || !dbRecord.getMethodName().equals(dbRecord.getMethodName())) {
            return ResponseEntity.ok(JSONObject.toJSONString(R.error("请求信息错误，请确认后再发送！")));
        }
        if (dbRecord.getFailedReason() == 0) {
            return ResponseEntity.ok(JSONObject.toJSONString(R.error("该请求参数不符合标准，无法进行重调，请校验参数后再申请！")));
        }
        if (dbRecord.getFailedTimes() > MAX_TIMES) {
            return ResponseEntity.ok(JSONObject.toJSONString(R.error("该请求已达到最大重调用次数，无法再重新请求!")));
        }
        RequestRecordEnum[] RequestRecordEnums = RequestRecordEnum.values();
        for (RequestRecordEnum recordenum : RequestRecordEnums) {
            if (recordenum.getMethodName().equals(dbRecord.getMethodName())) {
                if (recordenum.getPortName() == null) {
                    Method method = ReflectUtil.getMethod(recordenum.getControllerName(), recordenum.getMethodName());
                    Object invokeResponse;
                    try {
                        invokeResponse = method.invoke(snPushCheckRecordService, JSONObject.parseObject(dbRecord.getRequestDetail(), recordenum.getArgClass()));
                        updateRecord(dbRecord, 1, JSONObject.toJSONString(invokeResponse));

                        R data = new R(1000, "default success");
                        data.put("data", invokeResponse);
                        return ResponseEntity.ok(JSONObject.toJSONString(data));
                    } catch (Exception e) {
                        log.error("重调用失败，错误信息：{}", e);
                        updateRecord(dbRecord, 1);
                        e.printStackTrace();
                        if (recordenum.getMethodName().equals("singleCheckInvoice")) {
                            return ResponseEntity.ok(JSONObject.toJSONString(R.error("查验失败，错误信息：" + e.getMessage())));
                        }
                        return ResponseEntity.ok(JSONObject.toJSONString(R.error("推送失败，错误信息：" + e.getMessage())));
                    }
                }
            }
        }
        return ResponseEntity.ok(JSONObject.toJSONString(R.error("未找到相应信息！")));

    }


    private int updateRecord(RequestRecord record, Integer status, String... responseDetail) {
        DynamicContextHolder.push("business" + DbConstant.BUSINESS_WRITE);
        record.setLastRetryTime(new Date());
        record.setRetryTimes(record.getRetryTimes() + 1);
        record.setStatus(status);
        record.setRecallResponseDetail(responseDetail[0]);
        requestRecordDao.updateById(record);
        return requestRecordDao.updateById(record);
    }

}
